from .utils import download_from_remote, untar, RemoteFileMetadata, get_save_path


REMOTE = RemoteFileMetadata(
    filename='MAPS.tar',
    url='https://amubox.univ-amu.fr/s/iNG0xc5Td1Nv4rR/download',
    checksum=('9a8b89a7897b0ad95a505b4daa788302'),
)

DATASET_DIR = 'MAPS'


def download(data_home=None, force_overwrite=False):
    save_path = get_save_path(data_home)
    download_path = download_from_remote(REMOTE, force_overwrite=force_overwrite)
    untar(download_path, save_path)
    validate()


def validate():
    pass


def load():
    raise NotImplementedError()


def cite():
    raise NotImplementedError()



from __future__ import absolute_import

import os

import pytest

from mirdata import orchset, utils
from tests.test_utils import (mock_validated, mock_download, mock_unzip,
                              mock_validator, mock_force_delete_all)


@pytest.fixture
def mock_validate(mocker):
    return mocker.patch.object(orchset, 'validate')


@pytest.fixture
def data_home(tmpdir):
    return str(tmpdir)


@pytest.fixture
def mock_orchset_exists(mocker):
    return mocker.patch.object(os.path, 'exists')


def test_download_already_exists(data_home, mocker,
                                 mock_force_delete_all,
                                 mock_orchset_exists,
                                 mock_download,
                                 mock_unzip):
    mock_orchset_exists.return_value = True

    orchset.download(data_home)

    mock_force_delete_all.assert_not_called()
    mock_orchset_exists.assert_called_once()
    mock_download.assert_not_called()


def test_download_clean(data_home,
                        mocker,
                        mock_force_delete_all,
                        mock_download,
                        mock_unzip,
                        mock_orchset_exists):

    mock_orchset_exists.return_value = False
    mock_download.return_value = 'foobar'
    mock_unzip.return_value = ''

    orchset.download(data_home)

    mock_force_delete_all.assert_not_called()
    mock_orchset_exists.assert_called_once()
    mock_download.assert_called_once()
    mock_unzip.assert_called_once_with(mock_download.return_value, data_home)


def test_download_force_overwrite(data_home,
                          mocker,
                          mock_force_delete_all,
                          mock_orchset_exists,
                          mock_download,
                          mock_unzip):

    mock_orchset_exists.return_value = False
    mock_download.return_value = 'foobar'
    mock_unzip.return_value = ''

    orchset.download(data_home, force_overwrite=True)

    mock_force_delete_all.assert_called_once_with(orchset.REMOTE, data_home)
    mock_orchset_exists.assert_called_once()
    mock_download.assert_called_once()
    mock_unzip.assert_called_once_with(mock_download.return_value, data_home)


def test_validate_invalid(data_home, mocker, mock_validator):
    mock_validator.return_value = (True, True)

    missing_files, invalid_checksums = orchset.validate(data_home)
    assert missing_files and invalid_checksums
    mock_validator.assert_called_once()


def test_validate_valid(data_home, mocker, mock_validator):
    mock_validator.return_value = (False, False)

    missing_files, invalid_checksums = orchset.validate(data_home)
    assert not (missing_files or invalid_checksums)
    mock_validator.assert_called_once()



import pytest

import os
from time import sleep
import sys

"""
if sys.version_info[0] >= 3:
    import urllib.request
    from testcontainers.core.container import DockerContainer

    @pytest.fixture
    def file_server():
        container = DockerContainer('test-container:latest')
        container.with_bind_ports(8000, 8000)
        container.start()

        yield container

        container.stop()

    def test_sample(file_server, tmpdir):
        exposed_port = file_server.get_exposed_port(8000)

        # We do this because the web server in the container is not immediately available
        sleep(3)

        # This is the container we are running locally hence local IP
        url = 'http://127.0.0.1:%s' % exposed_port

        file_path = os.path.join(tmpdir, 'song1.mp3')

        urllib.request.urlretrieve(url, filename=file_path)
"""


def test_foobar():
    pass



# -*- coding: utf-8 -*-
"""Beatles Dataset Loader

The Beatles Dataset includes beat and metric position, chord, key, and segmentation
annotations for 179 Beatles songs. Details can be found in http://matthiasmauch.net/_pdf/mauch_omp_2009.pdf


Attributes:
    DATASET_DIR (str): The directory name for Beatles dataset. Set to `'Beatles'`.

    INDEX (dict): {track_id: track_data}.
        track_data is a jason data loaded from `index/`

    ANNOTATIONS_REMOTE (RemoteFileMetadata (namedtuple)): metadata
        of Beatles dataset. It includes the annotation file name, annotation
        file url, and checksum of the file.

"""
import csv
import os

try:
    from pathlib import Path
except ImportError:
    from pathlib2 import Path  # python 2 backport
import numpy as np

import mirdata.utils as utils

DATASET_DIR = 'Beatles'
INDEX = utils.load_json_index('beatles_index.json')
ANNOTATIONS_REMOTE = utils.RemoteFileMetadata(
    filename='The Beatles Annotations.tar.gz',
    url='http://isophonics.net/files/annotations/The%20Beatles%20Annotations.tar.gz',
    checksum='62425c552d37c6bb655a78e4603828cc',
)


class Track(object):
    """Beatles track class

    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        track_id (str): track id
        audio_path (str): track audio path
        title (str): title of the track
        beats (BeatData): beat annotation
        chords (ChordData): chords annotation
        key (KeyData): key annotation
        sections (SectionData): sections annotation

    """
    def __init__(self, track_id, data_home=None):
        if track_id not in INDEX:
            raise ValueError(
                '{} is not a valid track ID in Beatles'.format(track_id))

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = INDEX[track_id]

        self.audio_path = os.path.join(
            self._data_home, self._track_paths['audio'][0])

        self.title = os.path.basename(
            self._track_paths['sections'][0]).split('.')[0]

    @utils.cached_property
    def beats(self):
        return _load_beats(os.path.join(
            self._data_home, self._track_paths['beat'][0]))

    @utils.cached_property
    def chords(self):
        return _load_chords(os.path.join(
            self._data_home, self._track_paths['chords'][0]))

    @utils.cached_property
    def key(self):
        return _load_key(os.path.join(
            self._data_home, self._track_paths['keys'][0]))

    @utils.cached_property
    def sections(self):
        return _load_sections(os.path.join(
            self._data_home, self._track_paths['sections'][0]))


def download(data_home=None, force_overwrite=False):
    """Download the Beatles Dataset (annotations).
    The audio files are not provided due to the copyright.

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
        force_overwrite (bool): Whether to overwrite the existing downloaded data

    """

    # use the default location: ~/mir_datasets/Beatles
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    if os.path.exists(data_home) and not force_overwrite:
        return

    if force_overwrite:
        utils.force_delete_all(ANNOTATIONS_REMOTE, data_home=data_home)

    Path(data_home).mkdir(exist_ok=True)

    download_path = utils.download_from_remote(
        ANNOTATIONS_REMOTE, data_home=data_home, force_overwrite=force_overwrite
    )
    utils.untar(download_path, data_home, cleanup=True)

    missing_files, invalid_checksums = validate(data_home)
    if missing_files or invalid_checksums:
        print(
            """
            Unfortunately the audio files of the Beatles dataset are not available
            for download. If you have the Beatles dataset, place the contents into
            a folder called Beatles with the following structure:
                > Beatles/
                    > annotations/
                    > audio/
            and copy the Beatles folder to {}
        """.format(
                data_home
            )
        )


def validate(data_home=None):
    """Validate if a local version of this dataset is consistent

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths where the expected file exists locally
            but has a different checksum than the reference

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        INDEX, data_home
    )
    return missing_files, invalid_checksums


def track_ids():
    """Get the list of track IDs for this dataset

    Returns:
        (list): A list of track ids
    """
    return list(INDEX.keys())


def load(data_home=None):
    """Load Beatles dataset

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        (dict): {`track_id`: track data}

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    validate(data_home)
    beatles_data = {}
    for key in track_ids():
        beatles_data[key] = Track(key, data_home=data_home)
    return beatles_data


def _load_beats(beats_path):
    """Private function to load Beatles format beat data from a file

    Args:
        beats_path (str):

    """
    if beats_path is None or not os.path.exists(beats_path):
        return None

    beat_times, beat_positions = [], []
    with open(beats_path, 'r') as fhandle:
        dialect = csv.Sniffer().sniff(fhandle.read(1024))
        fhandle.seek(0)
        reader = csv.reader(fhandle, dialect)
        for line in reader:
            beat_times.append(float(line[0]))
            beat_positions.append(line[-1])

    beat_positions = _fix_newpoint(np.array(beat_positions))

    beat_data = utils.BeatData(np.array(beat_times), np.array(beat_positions))

    return beat_data


def _load_chords(chords_path):
    """Private function to load Beatles format chord data from a file

    Args:
        chords_path (str):

    """
    if chords_path is None or not os.path.exists(chords_path):
        return None

    start_times, end_times, chords = [], [], []
    with open(chords_path, 'r') as f:
        dialect = csv.Sniffer().sniff(f.read(1024))
        f.seek(0)
        reader = csv.reader(f, dialect)
        for line in reader:
            start_times.append(float(line[0]))
            end_times.append(float(line[1]))
            chords.append(line[2])

    chord_data = utils.ChordData(
        np.array(start_times), np.array(end_times), np.array(chords)
    )

    return chord_data


def _load_key(key_path):
    """Private function to load Beatles format key data from a file

    Args:
        key_path (str):

    """
    if key_path is None or not os.path.exists(key_path):
        return None

    start_times, end_times, keys = [], [], []
    with open(key_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter='\t')
        for line in reader:
            if line[2] == 'Key':
                start_times.append(float(line[0]))
                end_times.append(line[1])
                keys.append(line[3])

    key_data = utils.KeyData(np.array(start_times), np.array(end_times), np.array(keys))

    return key_data


def _load_sections(sections_path):
    """Private function to load Beatles format sections data from a file

    Args:
        sections_path (str):

    """
    if sections_path is None or not os.path.exists(sections_path):
        return None

    start_times, end_times, sections = [], [], []
    with open(sections_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter='\t')
        for line in reader:
            start_times.append(float(line[0]))
            end_times.append(line[1])
            sections.append(line[3])

    section_data = utils.SectionData(
        np.array(start_times), np.array(end_times), np.array(sections)
    )

    return section_data


def _fix_newpoint(beat_positions):
    """Fills in missing beat position labels by inferring the beat position
        from neighboring beats.

    """
    while np.any(beat_positions == 'New Point'):
        idxs = np.where(beat_positions == 'New Point')[0]
        for i in idxs:
            if i < len(beat_positions) - 1:
                if not beat_positions[i + 1] == 'New Point':
                    beat_positions[i] = str(np.mod(int(beat_positions[i + 1]) - 1, 4))
            if i == len(beat_positions) - 1:
                if not beat_positions[i - 1] == 'New Point':
                    beat_positions[i] = str(np.mod(int(beat_positions[i - 1]) + 1, 4))
    beat_positions[beat_positions == '0'] = '4'

    return beat_positions


def cite():
    """Print the reference"""

    cite_data = """
===========  MLA ===========

Mauch, Matthias, et al.
"OMRAS2 metadata project 2009."
10th International Society for Music Information Retrieval Conference (2009)

========== Bibtex ==========
@inproceedings{mauch2009beatles,
    title={OMRAS2 metadata project 2009},
    author={Mauch, Matthias and Cannam, Chris and Davies, Matthew and Dixon, Simon and Harte,
    Christopher and Kolozali, Sefki and Tidhar, Dan and Sandler, Mark},
    booktitle={12th International Society for Music Information Retrieval Conference},
    year={2009},
    series = {ISMIR}
}
    """

    print(cite_data)



# -*- coding: utf-8 -*-
"""SALAMI Dataset Loader

SALAMI Dataset.
Details can be found at http://ddmal.music.mcgill.ca/research/salami/annotations


Attributes:
    DIR (str): The directory name for SALAMI dataset. Set to `'Salami'`.

    INDEX (dict): {track_id: track_data}.
        track_data is a jason data loaded from `index/`

    METADATA (None): (todo?)

    ANNOT_REMOTE (RemoteFileMetadata (namedtuple)): metadata
        of SALAMI dataset. It includes the annotation file name, annotation
        file url, and checksum of the file.


"""
import csv
import numpy as np
import os
try:
    from pathlib import Path
except ImportError:
    from pathlib2 import Path  # python 2 backport

import mirdata.utils as utils

INDEX = utils.load_json_index('salami_index.json')
METADATA = None
DATASET_DIR = 'Salami'
ANNOTATIONS_REMOTE = utils.RemoteFileMetadata(
    filename='salami-data-public-master.zip',
    url='https://github.com/DDMAL/salami-data-public/archive/master.zip',
    checksum='b01d6eb5b71cca1f3163fae4b2cd4c61',
)


class Track(object):
    """SALAMI Track class

    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        track_id (str): track id
        audio_path (str): audio path of the track
        source
        annotator_1_id
        annotator_2_id
        duration_sec
        title
        artist
        annotator_1_time
        annotator_2-time
        broad_genre
        genre: (todo: somewhat separate metadata and annotation)
        sections_annotator_1_uppercase: annotation
        sections_annotator_1_lowercase: annotation
        sections_annotator_2_uppercase: annotation
        sections_annotator_2_lowercase: annotation


    """
    def __init__(self, track_id, data_home=None):
        if track_id not in INDEX:
            raise ValueError('{} is not a valid track ID in Salami'.format(track_id))

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = INDEX[track_id]

        if METADATA is None or METADATA['data_home'] != data_home:
            _reload_metadata(data_home)

        if track_id in METADATA.keys():
            self._track_metadata = METADATA[track_id]
        else:
            # annotations with missing metadata
            self._track_metadata = {
                'source': None,
                'annotator_1_id': None,
                'annotator_2_id': None,
                'duration_sec': None,
                'title': None,
                'artist': None,
                'annotator_1_time': None,
                'annotator_2_time': None,
                'class': None,
                'genre': None,
            }

        self.audio_path = os.path.join(
            self._data_home, self._track_paths['audio'][0])

        self.source = self._track_metadata['source']
        self.annotator_1_id = self._track_metadata['annotator_1_id']
        self.annotator_2_id = self._track_metadata['annotator_2_id']
        self.duration_sec = self._track_metadata['duration_sec']
        self.title = self._track_metadata['title']
        self.artist = self._track_metadata['artist']
        self.annotator_1_time = self._track_metadata['annotator_1_time']
        self.annotator_2_time = self._track_metadata['annotator_2_time']
        self.broad_genre = self._track_metadata['class']
        self.genre = self._track_metadata['genre']

    @utils.cached_property
    def sections_annotator_1_uppercase(self):
        return _load_sections(os.path.join(
            self._data_home, self._track_paths['annotator_1_uppercase']))

    @utils.cached_property
    def sections_annotator_1_lowercase(self):
        return _load_sections(os.path.join(
            self._data_home, self._track_paths['annotator_1_lowercase']))

    @utils.cached_property
    def sections_annotator_2_uppercase(self):
        return _load_sections(os.path.join(
            self._data_home, self._track_paths['annotator_2_uppercase']))

    @utils.cached_property
    def sections_annotator_2_lowercase(self):
        return _load_sections(os.path.join(
            self._data_home, self._track_paths['annotator_2_lowercase']))


def download(data_home=None, force_overwrite=False):
    """Download SALAMI Dataset (annotations).
    The audio files are not provided.

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

        force_overwrite (bool): whether to overwrite the existing downloaded data

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    if os.path.exists(data_home) and not force_overwrite:
        return

    if force_overwrite:
        utils.force_delete_all(ANNOTATIONS_REMOTE, data_home=data_home)

    Path(data_home).mkdir(exist_ok=True)

    download_path = utils.download_from_remote(
        ANNOTATIONS_REMOTE, data_home=data_home, force_overwrite=force_overwrite
    )

    utils.unzip(download_path, data_home, cleanup=True)
    missing_files, invalid_checksums = validate(data_home)
    if missing_files or invalid_checksums:
        print(
            """
            Unfortunately the audio files of the Salami dataset are not available
            for download. If you have the Salami dataset, place the contents into a
            folder called Salami with the following structure:
                > Salami/
                    > salami-data-public-master/
                    > audio/
            and copy the Salami folder to {}
        """.format(data_home)
        )


def validate(data_home=None):
    """Validate if the stored dataset is a valid version

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        INDEX, data_home
    )
    return missing_files, invalid_checksums


def track_ids():
    """Return track ids

    Returns:
        (list): A list of track ids
    """
    return list(INDEX.keys())


def load(data_home=None):
    """Load SALAMI dataset

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        (dict): {`track_id`: track data}

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    validate(data_home)
    salami_data = {}
    for key in track_ids():
        salami_data[key] = Track(key, data_home=data_home)
    return salami_data


def _load_sections(sections_path):
    if sections_path is None:
        return None

    times = []
    secs = []
    with open(sections_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter='\t')
        for line in reader:
            times.append(float(line[0]))
            secs.append(line[1])
    times = np.array(times)
    secs = np.array(secs)

    # remove sections with length == 0
    times_revised = np.delete(times, np.where(np.diff(times) == 0))
    secs_revised = np.delete(secs, np.where(np.diff(times) == 0))
    return utils.SectionData(
        np.array(times_revised[:-1]),
        np.array(times_revised)[1:],
        np.array(secs_revised)[:-1],
    )


def _load_metadata(data_home):

    metadata_path = os.path.join(
        data_home,
        os.path.join(
            'salami-data-public-master', 'metadata', 'metadata.csv'
        ),
    )

    if not os.path.exists(metadata_path):
        raise OSError('Could not find Salami metadata file')

    with open(metadata_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter=',')
        raw_data = []
        for line in reader:
            if line[0] == 'SONG ID':
                continue
            raw_data.append(line)

    metadata_index = {}
    for line in raw_data:
        track_id = line[0]

        metadata_index[track_id] = {
            'source': line[1],
            'annotator_1_id': line[2],
            'annotator_2_id': line[3],
            'duration_sec': line[5],
            'title': line[7],
            'artist': line[8],
            'annotator_1_time': line[10],
            'annotator_2_time': line[11],
            'class': line[14],
            'genre': line[15],
        }

    metadata_index['data_home'] = data_home

    return metadata_index


def _reload_metadata(data_home):
    global METADATA
    METADATA = _load_metadata(data_home=data_home)


def cite():
    """Print the reference"""

    cite_data = """
===========  MLA ===========
Smith, Jordan Bennett Louis, et al.,
"Design and creation of a large-scale database of structural annotations",
12th International Society for Music Information Retrieval Conference (2011)

========== Bibtex ==========
@inproceedings{smith2011salami,
    title={Design and creation of a large-scale database of structural annotations.},
    author={Smith, Jordan Bennett Louis and Burgoyne, John Ashley and
          Fujinaga, Ichiro and De Roure, David and Downie, J Stephen},
    booktitle={12th International Society for Music Information Retrieval Conference},
    year={2011},
    series = {ISMIR},
}
"""

    print(cite_data)



from __future__ import absolute_import

import os

import numpy as np
import pytest

from mirdata import beatles, utils
from tests.test_utils import (mock_validated, mock_download, mock_untar,
                              mock_validator, mock_force_delete_all)


@pytest.fixture
def mock_validate(mocker):
    return mocker.patch.object(beatles, 'validate')


@pytest.fixture
def data_home(tmpdir):
    return str(tmpdir)


@pytest.fixture
def mock_beatles_exists(mocker):
    return mocker.patch.object(os.path, 'exists')


def test_download_already_exists(data_home, mocker,
                                 mock_force_delete_all,
                                 mock_validator,
                                 mock_download,
                                 mock_untar):
    mock_beatles_exists.return_value = True

    beatles.download(data_home)

    mock_force_delete_all.assert_not_called()
    mock_download.assert_not_called()
    mock_untar.assert_not_called()
    mock_validator.assert_not_called()


def test_download_clean(data_home,
                        mocker,
                        mock_force_delete_all,
                        mock_beatles_exists,
                        mock_download,
                        mock_untar,
                        mock_validate):

    mock_beatles_exists.return_value = False
    mock_download.return_value = 'foobar'
    mock_untar.return_value = ''
    mock_validate.return_value = (False, False)

    beatles.download(data_home)

    mock_force_delete_all.assert_not_called()
    mock_beatles_exists.assert_called_once()
    mock_download.assert_called_once()
    mock_untar.assert_called_once_with(mock_download.return_value, data_home, cleanup=True)
    mock_validate.assert_called_once_with(data_home)


def test_download_force_overwrite(data_home,
                                  mocker,
                                  mock_force_delete_all,
                                  mock_beatles_exists,
                                  mock_download,
                                  mock_untar,
                                  mock_validate):

    mock_beatles_exists.return_value = False
    mock_download.return_value = 'foobar'
    mock_untar.return_value = ''
    mock_validate.return_value = (False, False)

    beatles.download(data_home, force_overwrite=True)

    mock_force_delete_all.assert_called_once_with(beatles.ANNOTATIONS_REMOTE, data_home=data_home)
    mock_beatles_exists.assert_called_once()
    mock_download.assert_called_once()
    mock_untar.assert_called_once_with(mock_download.return_value, data_home, cleanup=True)
    mock_validate.assert_called_once_with(data_home)


def test_validate_invalid(data_home, mocker, mock_validator):
    mock_validator.return_value = (True, True)

    missing_files, invalid_checksums = beatles.validate(data_home)
    assert missing_files and invalid_checksums
    mock_validator.assert_called_once()


def test_validate_valid(data_home, mocker, mock_validator):
    mock_validator.return_value = (False, False)

    missing_files, invalid_checksums = beatles.validate(data_home)
    assert not (missing_files or invalid_checksums)
    mock_validator.assert_called_once()


def test_track_ids():
    assert beatles.track_ids() == list(beatles.INDEX.keys())


def test_load_track_invalid_track_id():
    with pytest.raises(ValueError):
        beatles.Track('a-fake-track-id')


def test_fix_newpoint():
    beat_positions = np.asarray(['1', '2', 'New Point', '4'])

    actual = beatles._fix_newpoint(beat_positions)
    expected = np.asarray(['1', '2', '3', '4'])
    assert np.array_equal(actual, expected)



""" Setup script for mir_datasets. """
from setuptools import setup, find_packages

# import imp

# version = imp.load_source(
#     'mirdata.version', 'mirdata/version.py')

if __name__ == '__main__':
    setup(
        name='mirdata',
        version='0.0.5',
        description='Common loaders for MIR datasets.',
        url='https://github.com/mir-dataset-loaders/mirdata',
        packages=find_packages(exclude=['test', '*.test', '*.test.*']),
        download_url='http://github.com/mir-dataset-loaders/mirdata/releases',
        package_data={'mirdata': ['indexes/*']},
        long_description="""Common loaders for MIR datasets.""",
        keywords='mir dataset loader audio',
        license='BSD-3-Clause',
        install_requires=[
            'tqdm',
            'librosa<0.7.0',  # not using 0.7 for now because of sndfile
            'numpy>=1.16',
            'six',
        ],
        extras_require={
            'tests': [
                'pytest>=4.4.0',
                'pytest-cov>=2.6.1',
                'pytest-pep8>=1.0.0',
                'pytest-mock>=1.10.1',
                'pytest-localserver>=0.5.0',
                'testcontainers>=2.3',
                'future==0.17.1',
                'coveralls>=1.7.0',
            ],
            'docs': [
                'sphinx',
                'sphinxcontrib-napoleon',
                'sphinx_rtd_theme',
                'numpydoc',
            ],
        }
    )



from __future__ import absolute_import

import numpy as np

import pytest

from mirdata import ikala, utils


def test_track():
    data_home = 'tests/resources/mir_datasets/iKala'
    track = ikala.Track('10161_chorus', data_home=data_home)

    # test attributes are loaded as expected
    assert track.track_id == '10161_chorus'
    assert track._data_home == data_home
    assert track._track_paths == {
        "audio": [
            "Wavfile/10161_chorus.wav",
            "278ae003cb0d323e99b9a643c0f2eeda"
        ],
        "pitch": [
            "PitchLabel/10161_chorus.pv",
            "0d93a011a9e668fd80673049089bbb14"
        ],
        "lyrics": [
            "Lyrics/10161_chorus.lab",
            "79bbeb72b422056fd43be4e8d63319ce"
        ]
    }
    assert track.audio_path == 'tests/resources/mir_datasets/iKala/' + \
        'Wavfile/10161_chorus.wav'
    assert track.song_id == '10161'
    assert track.section == 'chorus'
    assert track.singer_id == '1'

    # test that cached properties don't fail and have the expected type
    assert type(track.f0) is utils.F0Data
    assert type(track.lyrics) is utils.LyricData

    # test audio loading functions
    vocal, sr_vocal = track.vocal_audio
    assert sr_vocal == 44100
    assert vocal.shape == (44100 * 2, )

    instrumental, sr_instrumental = track.instrumental_audio
    assert sr_instrumental == 44100
    assert instrumental.shape == (44100 * 2, )

    # make sure we loaded the correct channels to vocal/instrumental
    # (in this example, the first quarter second has only instrumentals)
    assert np.mean(np.abs(vocal[:8820])) < np.mean(np.abs(instrumental[:8820]))

    mix, sr_mix = track.mix_audio
    assert sr_mix == 44100
    assert mix.shape == (44100 * 2, )
    assert np.array_equal(mix, instrumental + vocal)


def test_track_ids():
    track_ids = ikala.track_ids()
    assert type(track_ids) is list
    assert len(track_ids) == 252


def test_load():
    data_home = 'tests/resources/mir_datasets/iKala'
    ikala_data = ikala.load(data_home=data_home, silence_validator=True)
    assert type(ikala_data) is dict
    assert len(ikala_data.keys()) is 252


def test_load_f0():
    # load a file which exists
    f0_path = 'tests/resources/mir_datasets/iKala/PitchLabel/10161_chorus.pv'
    f0_data = ikala._load_f0(f0_path)

    # check types
    assert type(f0_data) == utils.F0Data
    assert type(f0_data.times) is np.ndarray
    assert type(f0_data.frequencies) is np.ndarray
    assert type(f0_data.confidence) is np.ndarray

    # check values
    assert np.array_equal(f0_data.times, np.array([0.016, 0.048]))
    assert np.array_equal(f0_data.frequencies, np.array([0.0, 260.946404518887]))
    assert np.array_equal(f0_data.confidence, np.array([0.0, 1.0]))

    # load a file which doesn't exist
    f0_data_none = ikala._load_f0('fake/file/path')
    assert f0_data_none is None


def test_load_lyrics():
    # load a file without pronounciations
    lyrics_path_simple = 'tests/resources/mir_datasets/iKala/Lyrics/10161_chorus.lab'
    lyrics_data_simple = ikala._load_lyrics(lyrics_path_simple)

    #check types
    assert type(lyrics_data_simple) is utils.LyricData
    assert type(lyrics_data_simple.start_times) is np.ndarray
    assert type(lyrics_data_simple.end_times) is np.ndarray
    assert type(lyrics_data_simple.lyrics) is np.ndarray
    assert type(lyrics_data_simple.pronounciations) is np.ndarray

    # check values
    assert np.array_equal(lyrics_data_simple.start_times, np.array([0.027, 0.232]))
    assert np.array_equal(lyrics_data_simple.end_times, np.array([0.232, 0.968]))
    assert np.array_equal(lyrics_data_simple.lyrics, np.array(['JUST', 'WANNA']))
    assert np.array_equal(lyrics_data_simple.pronounciations, np.array([None, None]))

    # load a file with pronounciations
    lyrics_path_pronun = 'tests/resources/mir_datasets/iKala/Lyrics/10164_chorus.lab'
    lyrics_data_pronun = ikala._load_lyrics(lyrics_path_pronun)

    # check types
    assert type(lyrics_data_pronun) is utils.LyricData
    assert type(lyrics_data_pronun.start_times) is np.ndarray
    assert type(lyrics_data_pronun.end_times) is np.ndarray
    assert type(lyrics_data_pronun.lyrics) is np.ndarray
    assert type(lyrics_data_pronun.pronounciations) is np.ndarray

    # check values
    assert np.array_equal(lyrics_data_pronun.start_times, np.array([0.021, 0.571]))
    assert np.array_equal(lyrics_data_pronun.end_times, np.array([0.189, 1.415]))
    assert np.array_equal(lyrics_data_pronun.lyrics, np.array(['ASDF', 'EVERYBODY']))
    assert np.array_equal(lyrics_data_pronun.pronounciations, np.array(['t i au', None]))

    # load a file which doesn't exist
    lyrics_data_none = ikala._load_lyrics('fake/path')
    assert lyrics_data_none is None


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/iKala'
    metadata = ikala._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['10161'] == '1'
    assert metadata['21025'] == '1'


def test_cite():
    ikala.cite()



from __future__ import absolute_import

import os
import pytest
from mirdata import salami, utils
from tests.test_utils import (mock_download, mock_unzip,
                              mock_validator, mock_force_delete_all)


@pytest.fixture
def mock_validate(mocker):
    return mocker.patch.object(salami, 'validate')


@pytest.fixture
def mock_load_sections(mocker):
    return mocker.patch.object(salami, '_load_sections')


@pytest.fixture
def data_home(tmpdir):
    return str(tmpdir)


@pytest.fixture
def mock_salami_exists(mocker):
    return mocker.patch.object(os.path, 'exists')


def test_download_already_exists(data_home, mocker,
                                 mock_force_delete_all,
                                 mock_salami_exists,
                                 mock_validator,
                                 mock_download,
                                 mock_unzip):
    mock_salami_exists.return_value = True

    salami.download(data_home)

    mock_force_delete_all.assert_not_called()
    mock_salami_exists.assert_called_once()
    mock_download.assert_not_called()
    mock_unzip.assert_not_called()
    mock_validator.assert_not_called()


def test_download_clean(data_home,
                        mocker,
                        mock_force_delete_all,
                        mock_salami_exists,
                        mock_download,
                        mock_unzip,
                        mock_validate):

    mock_salami_exists.return_value = False
    mock_download.return_value = 'foobar'
    mock_unzip.return_value = ''
    mock_validate.return_value = (False, False)

    salami.download(data_home)

    mock_force_delete_all.assert_not_called()
    mock_salami_exists.assert_called_once()
    mock_download.assert_called_once()
    mock_unzip.assert_called_once_with(mock_download.return_value, data_home, cleanup=True)
    mock_validate.assert_called_once_with(data_home)


def test_download_force_overwrite(data_home,
                          mocker,
                          mock_force_delete_all,
                          mock_salami_exists,
                          mock_download,
                          mock_unzip,
                          mock_validate):

    mock_salami_exists.return_value = False
    mock_download.return_value = 'foobar'
    mock_unzip.return_value = ''
    mock_validate.return_value = (False, False)

    salami.download(data_home, force_overwrite=True)

    mock_force_delete_all.assert_called_once_with(salami.ANNOTATIONS_REMOTE, data_home=data_home)
    mock_salami_exists.assert_called_once()
    mock_download.assert_called_once()
    mock_unzip.assert_called_once_with(mock_download.return_value, data_home, cleanup=True)
    mock_validate.assert_called_once_with(data_home)


def test_validate_invalid(data_home, mocker, mock_validator):
    mock_validator.return_value = (True, True)

    missing_files, invalid_checksums = salami.validate(data_home)
    assert missing_files and invalid_checksums
    mock_validator.assert_called_once()


def test_validate_valid(data_home, mocker, mock_validator):
    mock_validator.return_value = (False, False)

    missing_files, invalid_checksums = salami.validate(data_home)
    assert not (missing_files or invalid_checksums)
    mock_validator.assert_called_once()


def test_load_track_invalid_track_id():
    with pytest.raises(ValueError):
        salami.Track('a_fake_track')


def test_load_track_no_metadata():
    with pytest.raises(OSError):
        salami.METADATA = None
        salami.Track('10')


def test_load_track_wrong_metadata():
    with pytest.raises(OSError):
        salami.METADATA = {'data_home': 'test'}
        salami.Track('10', 'wrong_data_home')


# def test_load_track_missing_metadata(mock_load_sections):
#     data_home = 'fake-data-home'
#     salami.METADATA = {'data_home': data_home}
#     mock_load_sections.return_value = ['a', 'b', 'c', 'd']

#     expected_track = salami.SalamiTrack(
#         '10',
#         'fake-data-home/Salami/audio/10.mp3',
#         'a',
#         'b',
#         'c',
#         'd',
#         None,
#         None,
#         None,
#         None,
#         None,
#         None,
#         None,
#         None,
#         None,
#         None,
#     )

#     actual_track = salami.load_track('10', data_home)
#     assert actual_track == expected_track


# def test_load_track_with_metadata(mock_load_sections):
#     track_metadata = {
#         'source': 'source', 'annotator_1_id': 'aid-1', 'annotator_2_id': 'aid-2',
#         'duration_sec': 60, 'title': 'title', 'artist': 'artist',
#         'annotator_1_time': None, 'annotator_2_time': None, 'class': 'class',
#         'genre': 'genre'
#     }
#     data_home = 'fake-data-home'
#     salami.METADATA = {'data_home': data_home, '10': track_metadata}
#     mock_load_sections.return_value = ['a', 'b', 'c', 'd']

#     expected_track = salami.SalamiTrack(
#         '10',
#         'fake-data-home/Salami/audio/10.mp3',
#         'a',
#         'b',
#         'c',
#         'd',
#         'source',
#         'aid-1',
#         'aid-2',
#         60,
#         'title',
#         'artist',
#         None,
#         None,
#         'class',
#         'genre',
#     )

#     actual_track = salami.load_track('10', data_home)
#     assert actual_track == expected_track


def test_track_ids():
    assert salami.track_ids() == list(salami.INDEX.keys())



# -*- coding: utf-8 -*-
"""iKala Dataset Loader

The iKala dataset is comprised of 252 30-second excerpts sampled from 206 iKala
songs (plus 100 hidden excerpts reserved for MIREX).
The music accompaniment and the singing voice are recorded at the left and right
channels respectively and can be found under the Wavfile directory.
In addition, the human-labeled pitch contours and timestamped lyrics can be
found under PitchLabel and Lyrics respectively.

Details can be found at http://mac.citi.sinica.edu.tw/ikala/


Attributes:
    DATASET_DIR (str): The directory name for iKala dataset. Set to `'iKala'`.

    INDEX (dict): {track_id: track_data}.
        track_data is a `IKalaTrack` namedtuple.

    TIME_STEP (float): Time step unit (in second) (TODO: what is this? hop length? window?)

    METADATA (None): TODO

    ID_MAPPING_URL (str): URL to get id-to-url mapping text file

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


import csv
import os
import librosa
import numpy as np

import mirdata.utils as utils

DATASET_DIR = 'iKala'
INDEX = utils.load_json_index('ikala_index.json')
TIME_STEP = 0.032  # seconds
METADATA = None
ID_MAPPING_URL = 'http://mac.citi.sinica.edu.tw/ikala/id_mapping.txt'


class Track(object):
    """iKala track class

    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        track_id (str): track id
        audio_path (str): track audio path
        song_id (str): song id of the track
        section (str): section (todo)
        singer_id (str): singer id
        f0 (F0Data): pitch
        lyrics (LyricData): lyrics

    """
    def __init__(self, track_id, data_home=None):
        if track_id not in INDEX:
            raise ValueError(
                '{} is not a valid track ID in iKala'.format(track_id))

        if METADATA is None or METADATA['data_home'] != data_home:
            _reload_metadata(data_home)

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = INDEX[track_id]

        self.audio_path = os.path.join(
            self._data_home, self._track_paths['audio'][0])
        self.song_id = track_id.split('_')[0]
        self.section = track_id.split('_')[1]
        if self.song_id in METADATA:
            self.singer_id = METADATA[self.song_id]
        else:
            self.singer_id = None

    @utils.cached_property
    def f0(self):
        return _load_f0(os.path.join(
            self._data_home, self._track_paths['pitch'][0]))

    @utils.cached_property
    def lyrics(self):
        return _load_lyrics(os.path.join(
            self._data_home, self._track_paths['lyrics'][0]))

    @property
    def vocal_audio(self):
        """Load iKala vocal audio

        Returns:
            vocal_channel (np.array): vocal audio. size of `(N, )`
            sr (int): sampling rate of the audio file
        """
        audio, sr = librosa.load(self.audio_path, sr=None, mono=False)
        vocal_channel = audio[1, :]
        return vocal_channel, sr

    @property
    def instrumental_audio(self):
        """Load iKala instrumental audio

        Returns:
            instrumental_channel (np.array): vocal audio. size of `(N, )`
            sr (int): sampling rate of the audio file
        """
        audio, sr = librosa.load(self.audio_path, sr=None, mono=False)
        instrumental_channel = audio[0, :]
        return instrumental_channel, sr

    @property
    def mix_audio(self):
        """Load iKala mixture audio

        Returns:
            mixed_audio (np.array): vocal audio. size of `(2, N)`
            sr (int): sampling rate of the audio file
        """
        mixed_audio, sr = librosa.load(self.audio_path, sr=None, mono=True)
        # multipy by 2 because librosa averages the left and right channel.
        return 2.0 * mixed_audio, sr


def download(data_home=None):
    """Download iKala Dataset. However, iKala dataset is not available for
    download anymore. This function prints a helper message to organize
    pre-downloaded iKala dataset.

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    print(
        """
        Unfortunately the iKala dataset is not available for download.
        If you have the iKala dataset, place the contents into a folder called
        {ikala_dir} with the following structure:
            > {ikala_dir}/
                > Lyrics/
                > PitchLabel/
                > Wavfile/
        and copy the {ikala_dir} folder to {save_path}
    """.format(
            ikala_dir=DATASET_DIR, save_path=data_home
        )
    )


def validate(data_home=None, silence=False):
    """Validate if the stored dataset is a valid version

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        INDEX, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Return track ids

    Returns:
        (list): A list of track ids
    """
    return list(INDEX.keys())


def load(data_home=None, silence_validator=False):
    """Load iKala dataset

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        (dict): {`track_id`: track data}

    """

    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    validate(data_home, silence=silence_validator)
    ikala_data = {}
    for key in INDEX.keys():
        ikala_data[key] = Track(key, data_home=data_home)
    return ikala_data


def _load_f0(f0_path):
    if not os.path.exists(f0_path):
        return None

    with open(f0_path) as fhandle:
        lines = fhandle.readlines()
    f0_midi = np.array([float(line) for line in lines])
    f0_hz = librosa.midi_to_hz(f0_midi) * (f0_midi > 0)
    confidence = (f0_hz > 0).astype(float)
    times = (np.arange(len(f0_midi)) * TIME_STEP) + (TIME_STEP / 2.0)
    f0_data = utils.F0Data(times, f0_hz, confidence)
    return f0_data


def _load_lyrics(lyrics_path):
    if not os.path.exists(lyrics_path):
        return None
    # input: start time (ms), end time (ms), lyric, [pronunciation]
    with open(lyrics_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter=' ')
        start_times = []
        end_times = []
        lyrics = []
        pronunciations = []
        for line in reader:
            start_times.append(float(line[0]) / 1000.0)
            end_times.append(float(line[1]) / 1000.0)
            lyrics.append(line[2])
            if len(line) > 2:
                pronunciation = ' '.join(line[3:])
                pronunciations.append(pronunciation if pronunciation != '' else None)
            else:
                pronunciations.append(None)

    lyrics_data = utils.LyricData(
        np.array(start_times), np.array(end_times),
        np.array(lyrics), np.array(pronunciations))
    return lyrics_data


def _reload_metadata(data_home):
    global METADATA
    METADATA = _load_metadata(data_home=data_home)


def _load_metadata(data_home):
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    id_map_path = os.path.join(data_home, 'id_mapping.txt')
    if not os.path.exists(id_map_path):
        utils.download_large_file(ID_MAPPING_URL, id_map_path)

    with open(id_map_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter='\t')
        singer_map = {}
        for line in reader:
            if line[0] == 'singer':
                continue
            singer_map[line[1]] = line[0]

    singer_map['data_home'] = data_home

    return singer_map


def cite():
    """Print the reference"""

    cite_data = """
=========== MLA ===========
Chan, Tak-Shing, et al.
"Vocal activity informed singing voice separation with the iKala dataset."
2015 IEEE International Conference on Acoustics, Speech and Signal Processing (ICASSP). IEEE, 2015.

========== Bibtex ==========
@inproceedings{chan2015vocal,
    title={Vocal activity informed singing voice separation with the iKala dataset},
    author={Chan, Tak-Shing and Yeh, Tzu-Chun and Fan, Zhe-Cheng and Chen, Hung-Wei and Su, Li and Yang, Yi-Hsuan and Jang, Roger},
    booktitle={2015 IEEE International Conference on Acoustics, Speech and Signal Processing (ICASSP)},
    pages={718--722},
    year={2015},
    organization={IEEE}
}
"""
    print(cite_data)



from __future__ import absolute_import

import os
import json

import pytest

from mirdata import medleydb_melody, utils
from tests.test_utils import mock_validated, mock_validator


@pytest.fixture
def mock_validate(mocker):
    return mocker.patch.object(medleydb_melody, 'validate')


@pytest.fixture
def data_home(tmpdir):
    return str(tmpdir)


def test_validate_invalid(data_home, mocker, mock_validator):
    mock_validator.return_value = (True, True)

    missing_files, invalid_checksums = medleydb_melody.validate(data_home)
    assert missing_files and invalid_checksums
    mock_validator.assert_called_once()


def test_validate_valid(data_home, mocker, mock_validator):
    mock_validator.return_value = (False, False)

    missing_files, invalid_checksums = medleydb_melody.validate(data_home)
    assert not (missing_files or invalid_checksums)
    mock_validator.assert_called_once()
